@@ -4,7 +4,7 @@ from fontTools.misc.textTools import safeEval
 from fontTools.misc.fixedTools import (
 	fixedToFloat as fi2fl, floatToFixed as fl2fi, ensureVersionIsLong as fi2ve,
 	versionToFixed as ve2fi)
-from .otBase import ValueRecordFactory
+from .otBase import ValueRecordFactory, CountReference
 from functools import partial
 import logging
 
@@ -23,7 +23,7 @@ def buildConverters(tableSpec, tableNamespace):
 		if name.startswith("ValueFormat"):
 			assert tp == "uint16"
 			converterClass = ValueFormat
-		elif name.endswith("Count") or name.endswith("LookupType"):
+		elif name.endswith("Count"):
 			assert tp in ("uint16", "uint32")
 			converterClass = ComputedUShort if tp == 'uint16' else ComputedULong
 		elif name == "SubTable":
@@ -207,8 +207,9 @@ class UInt24(IntValue):
 
 class ComputedInt(IntValue):
 	def xmlWrite(self, xmlWriter, font, value, name, attrs):
-		xmlWriter.comment("%s=%s" % (name, value))
-		xmlWriter.newline()
+		if value is not None:
+			xmlWriter.comment("%s=%s" % (name, value))
+			xmlWriter.newline()
 
 class ComputedUShort(ComputedInt, UShort):
 	pass
@@ -338,13 +339,38 @@ class Struct(BaseConverter):
 		Format = attrs.get("Format")
 		if Format is not None:
 			table.Format = int(Format)
+
+		noPostRead = not hasattr(table, 'postRead')
+		if noPostRead:
+			# TODO Cache table.hasPropagated.
+			cleanPropagation = False
+			for conv in table.getConverters():
+				if conv.isPropagated:
+					cleanPropagation = True
+					if not hasattr(font, '_propagator'):
+						font._propagator = {}
+					propagator = font._propagator
+					assert conv.name not in propagator
+					setattr(table, conv.name, None)
+					propagator[conv.name] = CountReference(table.__dict__, conv.name)
+
 		for element in content:
 			if isinstance(element, tuple):
 				name, attrs, content = element
 				table.fromXML(name, attrs, content, font)
 			else:
 				pass
-		# TODO Fill in items that are not set by XML.
+
+		if noPostRead:
+			table.populateDefaults(propagator=getattr(font, '_propagator', None))
+			if cleanPropagation:
+				for conv in table.getConverters():
+					if conv.isPropagated:
+						propagator = font._propagator
+						del propagator[conv.name]
+						if not propagator:
+							del font._propagator
+
 		return table
 
 	def __repr__(self):
